@@ -5,6 +5,7 @@ Constants for API library
 
 created 31-mar-2019 by richb@instantlinux.net
 """
+from flask_babel import _
 
 
 class Constants:
@@ -17,17 +18,14 @@ class Constants:
     MIME_VIDEO_TYPES = ("mp4", "mpeg")
     PER_PAGE_DEFAULT = 100
     REDIS_TTL = 3600
+    REGEX_EMAIL = r'[a-z0-9._%+-]+@[a-z0-9.-]+\.[a-z]{2,}'
+    REGEX_PHONE = r'[0-9()+ -]{5,20}'
     SERVICE_CONFIG_FILE = 'service_config.yaml'
 
 
-class i18n:
-    """
-    These are keys in a user-provided dict; see the
-    example/i18n_textstrings.py for how to assign string
-    templates for messaging.
-    """
+class i18nValues:
+    """These values exist only to provide msgids for generating messages.pot"""
 
-    # TODO - update this doc when messaging code is moved
-    # into this library
-    CONTACT_ADDED_REQUEST = 'contact_add'
-    PASSWORD_RESET = 'password_reset'
+    contact_types = dict(
+        email=_(u'email'), messenger=_(u'messenger'), sms=_(u'sms'),
+        voice=_(u'voice'), whatsapp=_(u'whatsapp'))
