@@ -1,10 +1,11 @@
 """service_registry.py
 
-Services or the UI discover one another through this service registry.
-Each microservice instance submits its identity and capabilities to
-this central registry, implemented as expiring redis keys which are
-updated at a fixed frequency. Encryption provides modest protection
-against injection attacks.
+Service Registry
+  Services or the UI discover one another through this service
+  registry.  Each microservice instance submits its identity and
+  capabilities to this central registry, implemented as expiring redis
+  keys which are updated at a fixed frequency. Encryption provides
+  modest protection against injection attacks.
 
 created 8-may-2020 by richb@instantlinux.net
 
@@ -12,11 +13,12 @@ created 8-may-2020 by richb@instantlinux.net
 
 import json
 import logging
-import redis
 import socket
 import threading
 
 from .aes_encrypt import AESEncrypt
+from .service_config import ServiceConfig
+from .session_manager import SessionManager
 from . import utils
 
 refresh_thread = None
@@ -25,25 +27,41 @@ params = {}
 
 
 class ServiceRegistry(object):
+    """
+    Service registry
 
-    def __init__(self, config, ttl=None, redis_conn=None):
+    Attributes:
+      db_session (obj): existing db session
+      ttl (int): how long to cache instance's registration
+      redis_conn (obj): connection to redis
+    """
+
+    def __init__(self, ttl=None, redis_conn=None):
         global params, refresh_thread
-        self.config = config
-        if not redis_conn and hasattr(config, 'redis_conn'):
-            redis_conn = config.redis_conn
+        self.config = config = ServiceConfig().config
+        if not redis_conn:
+            redis_conn = SessionManager().connection
         if not params:
             params = dict(
-                aes=AESEncrypt(self.config.REDIS_AES_SECRET),
-                connection=(
-                    redis_conn or redis.Redis(host=config.REDIS_HOST,
-                                              port=config.REDIS_PORT, db=0)),
+                aes=AESEncrypt(config.REDIS_AES_SECRET),
                 interval=config.REGISTRY_INTERVAL,
                 ttl=ttl or config.REGISTRY_TTL)
+        # TODO why doesn't this get initialized on first pass?
+        params['connection'] = redis_conn
         if not refresh_thread:
             refresh_thread = threading.Thread()
 
     def register(self, resource_endpoints, service_name=None,
                  instance_id=socket.gethostname(), tcp_port=None):
+        """register an instance serving a list of endpoints
+
+        Args:
+          resource_endpoints (list of str): controller endpoints served
+          service_name (str): microservice name
+          instance_id (str): unique ID of instance
+          tcp_port (int): port number of service
+        """
+
         global service_data
         try:
             ipv4 = socket.gethostbyname(instance_id)
@@ -51,7 +69,7 @@ class ServiceRegistry(object):
             ipv4 = None
         service_data['info'] = dict(
             endpoints=resource_endpoints, ipv4=ipv4,
-            port=tcp_port or self.config.PORT,
+            port=tcp_port or self.config.APP_PORT,
             public_url=self.config.PUBLIC_URL,
             created=utils.utcnow().replace(microsecond=0).isoformat())
         service_data['name'] = service_name or self.config.SERVICE_NAME
@@ -63,6 +81,11 @@ class ServiceRegistry(object):
 
     @staticmethod
     def update():
+        """background function to update registration at the defined
+        interval from local memory cache, until the instance
+        terminates.
+        """
+
         key = 'reg:%s:%s' % (service_data['name'], service_data['id'])
         try:
             params['connection'].set(key, params['aes'].encrypt(json.dumps(
@@ -76,7 +99,10 @@ class ServiceRegistry(object):
     def find(self, service_name=None):
         """ Finds one or all services
 
-        returns:
+        Args:
+          service_name (str): a service, or None for all
+
+        Returns:
           dict - instances (list of registered services)
                  url_map (public url for each top-level resource)
         """
